# pip install rich

import json
import sqlite3
import sys
from ipaddress import IPv4Network, IPv4Address, AddressValueError
from pathlib import Path

from rich.console import Console
from rich.table import Table

helps = """
    Команды:

    -h - вызов справки (пример: python3 ip2asn.py -h);
    -ip - поиск IP-адреса в базе (пример: python3 ip2asn.py -ip 192.168.1.1);
    -ipv - дополнительный параметр, для исключения поиска похожих данных 
                                (пример: python3 ip2asn.py -ipv 192.168.1.1);
    -d - поиск CIDR в базе (пример: python3 ip2asn.py -d 192.168.0.0/16);
    -a - поиск ASN в базе (пример: python3 ip2asn.py -a 3361);
    -c - выборка из базы по стране (пример: python3 ip2asn.py -c CN);
    -o - выборка из базы по оператору (пример: python3 ip2asn.py -o "BJ-GUANGHUAN-AP Beijing");
         обратите внимание, если название оператора пишется с пробелами, стоку необходимо заключить в кавычки.
    -oc - выборка по оператору и фильтр по стране (пример: python3 ip2asn.py -oc "BJ-GUANGHUAN-AP Beijing" CN);
    """


def select_data(column='', value='') -> (list, bool):
    path_base = Path.cwd() / 'ip2asn' / 'ip2asn_base.db'
    conn = sqlite3.connect(path_base)
    cur = conn.cursor()
    if not column:
        select_query = f"""SELECT * FROM ip2asn"""
        cur.execute(select_query)
    elif column == "owner":
        select_query = f"""SELECT * FROM ip2asn WHERE owner LIKE ?"""
        cur.execute(select_query, (f"%{value}%",))
    elif column in ['asn', 'country', 'cidr']:
        select_query = f"""SELECT * FROM ip2asn WHERE {column} = ?"""
        cur.execute(select_query, (value,))
    elif column == "asn,owner":
        data_list = []
        val = value.split("|")
        for col in ["asn", "owner"]:
            dat = []
            if col == "asn":
                select_query = f"""SELECT * FROM ip2asn WHERE {col} = ?"""
                dat = cur.execute(select_query, (val[0],))
            elif col == "owner":
                select_query = f"""SELECT * FROM ip2asn WHERE {col} LIKE ?"""
                dat = cur.execute(select_query, (f"%{val[1]}%",))
            if dat:
                data_list.extend(cur.fetchall())
        if data_list:
            return data_list
    data = cur.fetchall()
    cur.close()
    conn.close()
    return data if data else False


def ip_find(ip: str) -> (tuple, bool):
    for dat in select_data():
        if IPv4Address(ip) in IPv4Network(dat[0], False):
            return dat
    return False


def asn_country_cidr_find(column: str, value: str) -> (list, bool):
    if asn_list := select_data(column, value):
        return asn_list
    return False


def owner_find(value: str, country='') -> (list, bool):
    country_list = []
    if own := select_data('owner', value):
        if not country:
            return own
        else:
            for con in own:
                if con[2] == country:
                    country_list.append(con)
    if country_list:
        return country_list
    return False


def check_ip(ip):
    try:
        IPv4Address(ip)
        return True
    except AddressValueError:
        return False


def print_res(res: list, title: str):
    with open(Path.cwd() / 'ip2asn' / 'country_alpha2.json', 'r', encoding='utf-8') as cn:
        cn_code = json.load(cn)
    table = Table(title=title, title_justify='left')
    table.add_column("CIDR", no_wrap=False, justify="left", style="green")
    table.add_column("ASN", no_wrap=False, justify="left", style="green")
    table.add_column("Country", no_wrap=False, justify="left", style="green")
    table.add_column("Owner", no_wrap=False, justify="left", style="green")
    for r in res:
        table.add_row(r[0], r[1], f'"{cn_code[r[2]]}, {r[2]}"', f'"{r[3]}"')
    console = Console()
    print(' ')
    console.print(table)


if __name__ == "__main__":
    try:
        if sys.argv[1] == "-ip":
            if check_ip(sys.argv[2]):
                if fnd := ip_find(sys.argv[2]):
                    print_res([fnd], 'Найденные значения IP:')
                    if own := select_data('asn,owner', f'{fnd[2]}|{fnd[3]}'):
                        print_res(own, 'Дополнительные значения (ASN, владелец):')
                else:
                    print("Данные не найдены")
            else:
                print(helps)
        elif sys.argv[1] == "-ipv":
            if check_ip(sys.argv[2]):
                if fnd := ip_find(sys.argv[2]):
                    print_res([fnd], 'Найденные значения IP:')
                else:
                    print("Данные не найдены")
            else:
                print(helps)
        elif sys.argv[1] == "-d":
            try:
                IPv4Network(sys.argv[2])
            except AddressValueError:
                print(helps)
                sys.exit(0)
            if asn_country := asn_country_cidr_find('cidr', sys.argv[2]):
                print_res(asn_country, 'Найденные значения CIDR:')
            else:
                print("Данные не найдены")
        elif sys.argv[1] == "-a":
            if asn_country := asn_country_cidr_find('asn', sys.argv[2]):
                print_res(asn_country, 'Найденные значения ASN:')
            else:
                print("Данные не найдены")
        elif sys.argv[1] == "-c":
            if asn_country := asn_country_cidr_find('country', sys.argv[2]):
                print_res(asn_country, 'Найденные значения Country:')
            else:
                print("Данные не найдены")
        elif sys.argv[1] == "-o":
            if own := owner_find(sys.argv[2].replace("'", "").replace('"', '').replace("asn", "").replace("ASN", "")):
                print_res(own, 'Найденные значения Owner:')
            else:
                print("Данные не найдены")
        elif sys.argv[1] == "-oc":
            owner = sys.argv[2].replace("'", "").replace('"', '').replace("asn", "").replace("ASN", "")
            cont = sys.argv[3].upper()
            if own := owner_find(owner, cont):
                print_res(own, 'Найденные значения Owner - Country:')
            else:
                print("Данные не найдены")
        elif sys.argv[1] == "-h":
            print(helps)
    except IndexError:
        print(helps)
