# pip install pandas

from json import dump
from pathlib import Path
from ipaddress import IPv4Address
from sqlite3 import connect

from pandas import read_csv


def vacuum_base():
    """
    Очистка пустых значений из базы.
    """
    path_base = Path.cwd() / 'ip2asn' / 'ip2asn_base.db'
    conn = connect(path_base)
    conn.execute("VACUUM")
    conn.close()


def create_base():
    """
    Создаем базу данных sqlite.
    """
    path = Path.cwd() / 'ip2asn'
    path.mkdir(exist_ok=True)
    path_base = path / 'ip2asn_base.db'
    conn = connect(path_base)
    cur = conn.cursor()
    cur.execute("""CREATE TABLE IF NOT EXISTS ip2asn(
                   cidr TEXT,
                   asn TEXT,
                   country TEXT,
                   owner TEXT);
                """)
    conn.commit()
    cur.close()
    conn.close()


def save_to_base(data_list: list):
    """
    Сохраняем данные в базе.
    :param data_list: Список с кортежами из данных.
    """
    path_base = Path.cwd() / 'ip2asn' / 'ip2asn_base.db'
    conn = connect(path_base)
    cur = conn.cursor()
    cur.execute('BEGIN TRANSACTION')
    cur.executemany("INSERT INTO ip2asn VALUES(?, ?, ?, ?);", data_list)
    conn.commit()
    cur.close()
    conn.close()


def record_count() -> int:
    path_base = Path.cwd() / 'ip2asn' / 'ip2asn_base.db'
    conn = connect(path_base)
    cur = conn.cursor()
    select_query = f"""SELECT * FROM ip2asn"""
    cur.execute(select_query)
    data = len(cur.fetchall())
    cur.close()
    conn.close()
    return data


def ip_parse():
    asn_data = []
    ip2asn = dict()
    headers = ['start_ip', 'end_ip', 'asn', 'country', 'owner']
    df = read_csv(Path.cwd() / 'ip2asn' / 'ip2asn-v4-u32.tsv', sep='\t', header=None,
                  names=headers)
    start_ip = end_ip = asn = country = owner = ""
    for val in df.values:
        for num, item in enumerate(val):
            if df.columns[num] == "start_ip":
                start_ip = item
            elif df.columns[num] == "end_ip":
                end_ip = item
            elif df.columns[num] == "asn":
                asn = item
            elif df.columns[num] == "country":
                country = item
            elif df.columns[num] == "owner":
                owner = item
        prefixlen = 32 - ((int(end_ip) - int(start_ip) + 1).bit_length() - 1)
        cidr_notation = f'{IPv4Address(int(start_ip))}/{prefixlen}'
        print(f'CIDR: {cidr_notation} | ASN: {asn} | Country: {country} | Owner: {owner}')
        ip2asn.update({cidr_notation: {"asn": asn, "country": country, "owner": owner}})
        asn_data.append((cidr_notation, asn, country, owner))
    with open(Path.cwd() / 'ip2asn' / 'ip2asn-v4-u32.json', 'w', encoding='utf-8') as file:
        dump(ip2asn, file, indent=4, ensure_ascii=False)
    save_to_base(asn_data)


def main():
    if not (Path.cwd() / 'ip2asn' / 'ip2asn_base.db').exists():
        create_base()
    ip_parse()
    vacuum_base()
    print(f"\nКоличество записей в базе ASN: {record_count()}")


if __name__ == "__main__":
    main()
